module XhochY
  module Drossellog
    # Represents a line in a Apache combined-style logfile
    class LogLine
      # The whole line
      attr_reader :line
      # The IP which made the request
      attr_reader :ip
      # The day where the request was made 
      attr_reader :day
      # The month where the request was made
      attr_reader :month
      # The year where the request was made
      attr_reader :year
      # The hour where the request was made
      attr_reader :hour
      # The minute where the request was made
      attr_reader :minute
      # The second where the request was made
      attr_reader :second
      # The timezone of the server at the time of the request
      attr_reader :timezone
      # The method used to make the request
      attr_reader :method
      # The requested path
      attr_reader :path
      # The answered HTTP code form the server
      attr_reader :http_code
      # The size of the content sent back to the client
      attr_reader :size
      # The referrer which took the client to this point
      attr_reader :referrer
      # The requesting user agent
      attr_reader :agent
      
      # The regular expression to parse a apache logfile line
      @@rxp = /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s([^\s]+)\s([^\s]+)\s\[(\d{1,2})\/([^\/]+)\/(\d{4})\:(\d{1,2})\:(\d{1,2})\:(\d{1,2})\s([+-]\d{4})\]\s\"([A-Z]+)\s([^\s]+)[^\"]+\"\s(\d+)\s(\d+|-)\s\"([^\"]*)\"\s\"([^\"]*)\"/
    
      # check if the line is a valid apache-combined logfile line
      def self.matches?(line)
        ('line' =~ @@rxp)
      end
      
      # return the regular expression for parsing a apache-combined logfile line
      def self.get_rxp
        @@rxp
      end
      
       # (re)initialize the object with data from a hash
      def init_with_hash(hash)
        @line = hash['line']
        @day = hash['day']
        @ip = hash['ip']
        @month = hash['month']
        @year = hash['year']
        @hour = hash['hour']
        @minute = hash['minute']
        @second = hash['second']
        @timezone = hash['timezone']
        @method = hash['method']
        @path = hash['path']
        @http_code = hash['http_code']
        @size = hash['size']
        @referrer = hash['referrer']
        @agent = hash['agent']
      end #^ init_with_hash

      # (re)initialize the object with data from a string
      def init_with_string(line)
        @line = line
        matches = @@rxp.match(line)
        @ip = matches[1]
        # matches[2] -> user ..
        # matches[3] -> user ..
        @day = matches[4].to_i
        @month = matches[5]
        @year = matches[6].to_i
        @hour = matches[7].to_i
        @minute = matches[8].to_i
        @second = matches[9].to_i
        @timezone = matches[10]
        @method = matches[11]
        @path = matches[12] 
        @http_code = matches[13]
        @size = matches[14].to_i
        @referrer = matches[15]
        @agent = matches[16]
      end #^ init_with_string
      
      # Returns this line as a Hash object
      def to_hash(strip_tags)
        hash = {}
        
        if !strip_tags.include?('ip')
          hash['ip'] = @ip
        end
        
        if !strip_tags.include?('day')
          hash['day'] = @day
        end
        
        if !strip_tags.include?('month')
          hash['month'] = @month
        end
        
        if !strip_tags.include?('year')
          hash['year'] = @year
        end
        
        if !strip_tags.include?('hour')
          hash['hour'] = @hour
        end
        
        if !strip_tags.include?('minute')
          hash['minute'] = @minute
        end
        
        if !strip_tags.include?('second')
          hash['second'] = @second
        end
        
        if !strip_tags.include?('timezone')
          hash['timezone'] = @timezone
        end
        
        if !strip_tags.include?('method')
          hash['method'] = @method
        end
        
        if !strip_tags.include?('path')
          hash['path'] = @path
        end
        
        if !strip_tags.include?('http_code')
          hash['http_code'] = @http_code
        end
        
        if !strip_tags.include?('size')
          hash['size'] = @size
        end
        
        if !strip_tags.include?('referrer')
          hash['referrer'] = @referrer
        end
        
        if !strip_tags.include?('agent')
          hash['agent'] = @agent
        end
        
        hash
      end #^ to_hash
      
       # Create a new instance of a XYLogLine object
      def initialize(arg)
        if arg.class == String
          init_with_string arg
        end
        if arg.class == Hash 
          init_with_hash arg
        end
      end #^ initialize
      
    end #^ LogLine
  end #^ Drossellog
end #^ XhochY
